'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

require('isomorphic-fetch');

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var mimeLib = {
  json: 'application/json',
  form: 'multipart/form-data',
  html: 'text/html',
  xml: 'application/xml',
  text: 'text/plain'
};

var Hifetch = function () {
  function Hifetch(opt) {
    _classCallCheck(this, Hifetch);

    this._config = Object.assign({}, Hifetch.defaultConfig, opt);
    var _config = this._config,
        query = _config.query,
        method = _config.method,
        responseType = _config.responseType,
        data = _config.data,
        originDataType = _config.dataType,
        FormData = _config.FormData,
        headers = _config.headers,
        jwtToken = _config.jwtToken,
        enableCookies = _config.enableCookies,
        disableCORS = _config.disableCORS;

    this._queryString = query ? _qs2.default.stringify(query) : '';
    var fetchOpt = this._fetchOpt = {};
    if (disableCORS) {
      fetchOpt.mode = 'cors';
    }
    if (enableCookies) {
      fetchOpt.credentials = disableCORS ? 'same-origin' : 'include';
    }
    var moreHeaders = {
      Accept: /\//.test(responseType) ? responseType : mimeLib[responseType]
    };
    if (jwtToken) {
      moreHeaders['Authorization'] = 'Bearer ' + jwtToken;
    }
    var dataType = void 0;
    var dataForBody = data;
    var mergeToForm = function mergeToForm(dict) {
      Object.keys(dict).forEach(function (key) {
        dataForBody.append(key, dict[key]);
      });
    };
    if (data) {
      dataType = (/\//.test(originDataType) ? originDataType : mimeLib[originDataType]) || Hifetch.defaultConfig.dataType;
      if ((typeof data === 'undefined' ? 'undefined' : _typeof(data)) === 'object') {
        if (FormData && data instanceof FormData) {
          dataType = mimeLib.form;
        } else if (data.constructor !== Object) {
          throw new Error('hifetch: `data` must be a plain object, FormData object or string');
        } else if (dataType === mimeLib.form) {
          dataForBody = new FormData();
          mergeToForm(data);
        } else if (dataType === Hifetch.defaultConfig.dataType) {
          dataForBody = _qs2.default.stringify(data);
        } else if (dataType.indexOf('json') !== -1) {
          dataForBody = JSON.stringify(data);
        }
      }
    }
    Object.assign(fetchOpt, {
      method: method.toUpperCase(),
      headers: Object.assign({}, headers, moreHeaders, !dataType || dataType === mimeLib.form ? {
        // leave Content-Type' field empty, Fetch APi will automatically create:
        // Content-Type: multipart/form-data; boundary=----WebKitFormBoundaryyEmKNDsBKjB7QEqu
      } : {
        'Content-Type': dataType
      })
    }, data ? {
      body: dataForBody
    } : {});
  }

  _createClass(Hifetch, [{
    key: 'send',
    value: function send() {
      var _this = this;

      var _config2 = this._config,
          url = _config2.url,
          timeout = _config2.timeout,
          validateStatus = _config2.validateStatus,
          mergeHeaders = _config2.mergeHeaders,
          parser = _config2.parser,
          handler = _config2.handler,
          success = _config2.success,
          error = _config2.error;

      var current = this._current = uuid();
      return new Promise(function (resolve, reject) {
        if (timeout) {
          setTimeout(function () {
            if (current === _this._current) {
              reject('timeout');
            }
          }, timeout);
        }
        return fetch(_this._queryString ? url + '?' + _this._queryString : url, _this._fetchOpt).then(function (res) {
          if (current === _this._current) {
            resolve(res);
          }
        }).catch(function (res) {
          if (current === _this._current) {
            reject(res);
          }
        });
      }).then(function (response) {
        if (validateStatus(response.status)) {
          return response;
        }
        var err = new Error(response.statusText);
        err.response = response;
        throw err;
      }).then(function (response) {
        var headers = {};
        Object.keys(mergeHeaders).forEach(function (key) {
          var header = mergeHeaders[key];
          if (header) {
            headers[key] = response.headers.get(header);
          }
        });
        var withHeaders = function withHeaders(data) {
          return {
            data: data,
            headers: headers
          };
        };
        var parsed = parser(response);
        if (parsed.then) {
          return parsed.then(withHeaders);
        }
        return withHeaders(parsed);
      }).then(function (_ref) {
        var data = _ref.data,
            headers = _ref.headers;

        try {
          if ((typeof data === 'undefined' ? 'undefined' : _typeof(data)) === 'object' && data.status) {
            return error(Object.assign(headers, {
              status: 3,
              message: '[REMOTE ERROR] status: ' + data.status + ' message: ' + data.message,
              data: data
            }));
          }
          var result = handler ? handler(data, headers) : typeof data !== 'string' && Object.assign(headers, data) || data;
          return success(result);
        } catch (err) {
          return error({
            status: 4,
            message: '[CUSTOM JS ERROR] ' + clearErrorMessage(err),
            error: err
          });
        }
      }).catch(function (err) {
        if (err === 'timeout') {
          return error({
            status: 5,
            message: '[TIMEOUT ERROR] limit: ' + timeout / 1000 + 's',
            timeout: timeout
          });
        }
        if (err.status) {
          return error(err);
        }

        var _ref2 = err.response || {},
            httpStatus = _ref2.status,
            httpStatusText = _ref2.statusText;

        if (httpStatus) {
          return error({
            status: 2,
            message: '[FETCH ERROR] ' + httpStatus + ' ' + httpStatusText,
            httpStatus: httpStatus,
            httpStatusText: httpStatusText
          });
        }
        return error({
          status: 1,
          message: '[INTERNAL JS ERROR] ' + clearErrorMessage(err),
          error: err
        });
      });
    }
  }, {
    key: 'cancel',
    value: function cancel() {
      this._current = null;
    }
  }, {
    key: 'error',
    value: function error(customError) {
      var error = this._config.error;

      this._current = null;
      return error({
        status: 6,
        message: '[MANUAL ERROR] ' + clearErrorMessage(customError),
        customError: customError
      });
    }
  }]);

  return Hifetch;
}();

Hifetch.defaultConfig = {
  url: '',
  method: 'get',
  query: null,
  data: '',
  dataType: 'application/x-www-form-urlencoded',
  FormData: typeof window !== 'undefined' ? window.FormData : null,
  jwtToken: '',
  headers: {},
  mergeHeaders: {},
  enableCookies: false,
  disableCORS: false,
  validateStatus: function validateStatus(status) {
    return status >= 200 && status < 300;
  },
  parser: function parser(response) {
    return response.json();
  },

  responseType: 'application/json',
  timeout: 0,
  handler: null,
  success: function success(res) {
    return res;
  },
  error: function error(res) {
    return Promise.reject(res);
  }
};


function uuid() {
  return new Date().getTime() * 1000 + Math.floor(Math.random() * 1000);
}

function clearErrorMessage(err) {
  return err.toString(); // .replace(/.+:\s*/, '');
}

function hifetch(opt) {
  return new Hifetch(opt);
}

hifetch.Class = Hifetch;

exports.default = hifetch;
module.exports = exports['default'];
//# sourceMappingURL=index.js.map