'use strict';

var _chai = require('chai');

var _nock = require('nock');

var _nock2 = _interopRequireDefault(_nock);

var _dist = require('../dist');

var _dist2 = _interopRequireDefault(_dist);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-unused-expressions: 0 */
describe('get', function () {

  it('user1, promise style', function (done) {
    (0, _nock2.default)(_utils.ROOT, {
      reqheaders: {
        'X-My-Headers': '1'
      }
    }).defaultReplyHeaders({
      'X-Powered-By': 'nock'
    }).get('/user').query({
      uid: 1
    }).reply(200, {
      name: 'user1'
    });
    (0, _dist2.default)({
      url: _utils.ROOT + '/user',
      query: {
        uid: 1
      },
      headers: {
        'X-My-Headers': '1'
      },
      mergeHeaders: {
        poweredBy: 'X-Powered-By'
      }
    }).send().then(function (res) {
      (0, _chai.expect)(res.name).to.be.equal('user1');
      (0, _chai.expect)(res.poweredBy).to.be.equal('nock');
      done();
    }).catch(function (err) {
      console.log(err);
    });
  });

  it('user1, callback style', function (done) {
    (0, _nock2.default)(_utils.ROOT, {
      reqheaders: {
        'X-My-Headers': '1'
      }
    }).defaultReplyHeaders({
      'X-Powered-By': 'nock'
    }).get('/user').query({
      uid: 1
    }).reply(200, {
      name: 'user1'
    });
    (0, _dist2.default)({
      url: _utils.ROOT + '/user',
      query: {
        uid: 1
      },
      headers: {
        'X-My-Headers': '1'
      },
      mergeHeaders: {
        poweredBy: 'X-Powered-By'
      },
      success: function success(res) {
        (0, _chai.expect)(res.name).to.be.equal('user1');
        (0, _chai.expect)(res.poweredBy).to.be.equal('nock');
        return res;
      }
    }).send().then(function (res) {
      (0, _chai.expect)(res.name).to.be.equal('user1');
      (0, _chai.expect)(res.poweredBy).to.be.equal('nock');
      done();
    }).catch(function (err) {
      console.log(err);
    });
  });

  it('user2', function (done) {
    (0, _nock2.default)(_utils.ROOT, {
      reqheaders: {
        'X-My-Headers': '1'
      }
    }).get('/user').query({
      uid: 2
    }).reply(200, {
      name: 'user2'
    });
    (0, _dist2.default)({
      url: _utils.ROOT + '/user',
      query: {
        uid: 2
      },
      headers: {
        'X-My-Headers': '1'
      },
      success: function success(res) {
        (0, _chai.expect)(res.name).to.be.equal('user2');
        return res;
      }
    }).send().then(function (res) {
      (0, _chai.expect)(res.name).to.be.equal('user2');
      done();
    }).catch(function (err) {
      console.log(err);
    });
  });

  it('text', function (done) {
    (0, _nock2.default)(_utils.ROOT, {
      reqheaders: {
        'X-My-Headers': '1'
      }
    }).defaultReplyHeaders({
      'X-Powered-By': 'nock'
    }).get('/text').reply(200, 'some text');
    (0, _dist2.default)({
      url: _utils.ROOT + '/text',
      headers: {
        'X-My-Headers': '1'
      },
      parser: function parser(response) {
        return response.text();
      },
      mergeHeaders: {
        poweredBy: 'X-Powered-By'
      }
    }).send().then(function (res) {
      (0, _chai.expect)(res).to.be.equal('some text');
      done();
    }).catch(function (err) {
      console.log(err);
    });
  });
});
//# sourceMappingURL=get.spec.js.map